# pylint doesn't know about pytest fixtures


import json
import os
import re
import time

import docker
import pytest
from dagster._core.storage.dagster_run import DagsterRunStatus, RunsFilter
from dagster._core.test_utils import environ, poll_for_finished_run, poll_for_step_start
from dagster_docker.docker_run_launcher import (
    DOCKER_CONTAINER_ID_TAG,
    DOCKER_IMAGE_TAG,
    DockerRunLauncher,
)
from dagster_shared.yaml_utils import merge_yamls
from dagster_test.test_project import (
    ReOriginatedExternalJobForTest,
    find_local_test_image,
    get_buildkite_registry_config,
    get_test_project_docker_image,
    get_test_project_environments_path,
    get_test_project_recon_job,
    get_test_project_workspace_and_remote_job,
)

from dagster_docker_tests import IS_BUILDKITE


@pytest.mark.integration
def test_launch_docker_no_network(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()
    launcher_config = {"env_vars": aws_env}

    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_yamls(
        [
            os.path.join(get_test_project_environments_path(), "env.yaml"),
            os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
        ]
    )
    with docker_postgres_instance(
        overrides={
            "run_launcher": {
                "class": "DockerRunLauncher",
                "module": "dagster_docker",
                "config": launcher_config,
            }
        },
    ) as instance:
        recon_job = get_test_project_recon_job("demo_job_s3", docker_image)
        with get_test_project_workspace_and_remote_job(
            instance, "demo_job_s3", container_image=docker_image
        ) as (workspace, orig_job):
            remote_job = ReOriginatedExternalJobForTest(
                orig_job,
                container_image=docker_image,
            )
            run = instance.create_run_for_job(
                job_def=recon_job.get_definition(),
                run_config=run_config,
                remote_job_origin=remote_job.get_remote_origin(),
                job_code_origin=remote_job.get_python_origin(),
            )
            instance.launch_run(run.run_id, workspace)

            # Container launches, but run is stuck in STARTING state
            # due to not being able to access the network
            run = instance.get_run_by_id(run.run_id)
            assert run.tags[DOCKER_IMAGE_TAG] == docker_image

            container_id = run.tags[DOCKER_CONTAINER_ID_TAG]

            run = instance.get_run_by_id(run.run_id)

            assert run.status == DagsterRunStatus.STARTING
            assert run.tags[DOCKER_IMAGE_TAG] == docker_image
            client = docker.client.from_env()

            container = None

            try:
                start_time = time.time()
                while True:
                    container = client.containers.get(container_id)
                    if time.time() - start_time > 60:
                        raise Exception("Timed out waiting for container to exit")

                    if container.status == "exited":
                        break

                    time.sleep(3)

            finally:
                if container:
                    container.remove(force=True)


@pytest.mark.integration
def test_launch_docker_image_on_job_config(docker_postgres_instance, aws_env):
    # Docker image name to use for launch specified as part of the job origin
    # rather than in the run launcher instance config

    docker_image = get_test_project_docker_image()
    launcher_config = {
        "env_vars": aws_env + ["DOCKER_LAUNCHER_NETWORK"],
        "network": {"env": "DOCKER_LAUNCHER_NETWORK"},
        "container_kwargs": {
            "auto_remove": False,
            "labels": {"foo": "baz", "bar": ""},
        },
    }

    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_yamls(
        [
            os.path.join(get_test_project_environments_path(), "env.yaml"),
            os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
        ]
    )

    with environ({"DOCKER_LAUNCHER_NETWORK": "container:postgres"}):
        with docker_postgres_instance(
            overrides={
                "run_launcher": {
                    "class": "DockerRunLauncher",
                    "module": "dagster_docker",
                    "config": launcher_config,
                }
            }
        ) as instance:
            recon_job = get_test_project_recon_job("demo_job_s3", docker_image)
            with get_test_project_workspace_and_remote_job(
                instance, "demo_job_s3", container_image=docker_image
            ) as (workspace, orig_job):
                remote_job = ReOriginatedExternalJobForTest(
                    orig_job,
                    container_image=docker_image,
                )
                run = instance.create_run_for_job(
                    job_def=recon_job.get_definition(),
                    run_config=run_config,
                    remote_job_origin=remote_job.get_remote_origin(),
                    job_code_origin=remote_job.get_python_origin(),
                )
                instance.launch_run(run.run_id, workspace)

                poll_for_finished_run(instance, run.run_id, timeout=60)

                run = instance.get_run_by_id(run.run_id)

                assert run.status == DagsterRunStatus.SUCCESS

                assert run.tags[DOCKER_IMAGE_TAG] == docker_image

                container_obj = instance.run_launcher._get_container(run)  # noqa
                assert container_obj.labels["foo"] == "baz"
                assert container_obj.labels["bar"] == ""
                assert container_obj.labels["dagster/run_id"] == run.run_id
                assert container_obj.labels["dagster/job_name"] == run.job_name


def check_event_log_contains(event_log, expected_type_and_message):
    types_and_messages = [
        (e.dagster_event.event_type_value, e.message) for e in event_log if e.is_dagster_event
    ]

    for expected_event_type, expected_message_fragment in expected_type_and_message:
        assert any(
            event_type == expected_event_type and expected_message_fragment in message
            for event_type, message in types_and_messages
        )


@pytest.mark.integration
def test_terminate_launched_docker_run(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()
    launcher_config = {
        "env_vars": aws_env,
        "network": "container:postgres",
        "container_kwargs": {"stop_timeout": 15},
    }

    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = {
        **merge_yamls(
            [
                os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
            ]
        ),
        "ops": {"hanging_op": {"config": {"cleanup_delay": 10}}},
    }

    with docker_postgres_instance(
        overrides={
            "run_launcher": {
                "class": "DockerRunLauncher",
                "module": "dagster_docker",
                "config": launcher_config,
            }
        }
    ) as instance:
        recon_job = get_test_project_recon_job("hanging_job", docker_image)
        with get_test_project_workspace_and_remote_job(
            instance, "hanging_job", container_image=docker_image
        ) as (workspace, orig_job):
            remote_job = ReOriginatedExternalJobForTest(
                orig_job,
                container_image=docker_image,
            )

            run = instance.create_run_for_job(
                job_def=recon_job.get_definition(),
                run_config=run_config,
                remote_job_origin=remote_job.get_remote_origin(),
                job_code_origin=remote_job.get_python_origin(),
            )

            run_id = run.run_id

            instance.launch_run(run_id, workspace)

            poll_for_step_start(instance, run_id)

            assert instance.run_launcher.terminate(run_id)

            terminated_run = poll_for_finished_run(instance, run_id, timeout=30)
            terminated_run = instance.get_run_by_id(run_id)
            assert terminated_run.status == DagsterRunStatus.CANCELED
            run_logs = instance.all_logs(run_id)

            check_event_log_contains(
                run_logs,
                [
                    ("PIPELINE_CANCELING", "Sending run termination request"),
                    ("STEP_FAILURE", 'Execution of step "hanging_op" failed.'),
                    ("PIPELINE_CANCELED", 'Execution of run for "hanging_job" canceled.'),
                    ("ENGINE_EVENT", "Process for run exited"),
                ],
            )


@pytest.mark.integration
def test_launch_docker_invalid_image(docker_postgres_instance, aws_env):
    docker_image = "_invalid_format_image"
    launcher_config = {
        "env_vars": aws_env,
        "network": "container:postgres",
        "image": docker_image,
    }

    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()

    run_config = merge_yamls(
        [
            os.path.join(get_test_project_environments_path(), "env.yaml"),
            os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
        ]
    )

    with docker_postgres_instance(
        overrides={
            "run_launcher": {
                "class": "DockerRunLauncher",
                "module": "dagster_docker",
                "config": launcher_config,
            }
        }
    ) as instance:
        recon_job = get_test_project_recon_job("demo_job_s3")
        with get_test_project_workspace_and_remote_job(instance, "demo_job_s3") as (
            workspace,
            orig_job,
        ):
            remote_job = ReOriginatedExternalJobForTest(orig_job)

            run = instance.create_run_for_job(
                job_def=recon_job.get_definition(),
                run_config=run_config,
                remote_job_origin=remote_job.get_remote_origin(),
                job_code_origin=remote_job.get_python_origin(),
            )

            with pytest.raises(
                Exception,
                match=re.escape(
                    "Docker image name _invalid_format_image is not correctly formatted"
                ),
            ):
                instance.launch_run(run.run_id, workspace)


@pytest.mark.integration
def test_launch_docker_image_on_instance_config(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()
    launcher_config = {
        "env_vars": aws_env,
        "network": "container:postgres",
        "image": docker_image,
    }

    _test_launch(docker_postgres_instance, docker_image, launcher_config)


@pytest.mark.integration
def test_launch_docker_image_multiple_networks(docker_postgres_instance, postgres_network, aws_env):
    docker_image = get_test_project_docker_image()
    launcher_config = {
        "env_vars": aws_env,
        "networks": [
            "container:postgres",
            postgres_network,
        ],
        "image": docker_image,
    }
    _test_launch(docker_postgres_instance, docker_image, launcher_config)


@pytest.mark.integration
def test_launch_docker_config_on_container_context(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()
    launcher_config = {}
    _test_launch(
        docker_postgres_instance,
        docker_image,
        launcher_config,
        container_image=docker_image,
        container_context={
            "docker": {
                "env_vars": aws_env,
                "networks": [
                    "container:postgres",
                ],
            }
        },
    )


@pytest.mark.integration
def test_cant_combine_network_and_networks(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()
    launcher_config = {
        "env_vars": aws_env,
        "network": "container:postgres",
        "networks": [
            "postgres",
        ],
        "image": docker_image,
    }
    with pytest.raises(Exception, match="cannot set both `network` and `networks`"):
        with docker_postgres_instance(
            overrides={
                "run_launcher": {
                    "class": "DockerRunLauncher",
                    "module": "dagster_docker",
                    "config": launcher_config,
                }
            }
        ) as instance:
            print(instance.run_launcher)  # noqa: T201


from unittest import mock

from dagster._core.launcher.base import WorkerStatus
from dagster._core.test_utils import create_run_for_test, instance_for_test


def test_check_run_health():
    mock_container_state = {
        "Status": "exited",
        "Running": False,
        "Paused": False,
        "Restarting": False,
        "OOMKilled": True,
        "Dead": False,
        "Pid": 0,
        "ExitCode": 1,
        "Error": "Out of memory",
        "StartedAt": "2024-09-16T12:49:45.539998202Z",
        "FinishedAt": "2024-09-16T13:00:00.000000000Z",
    }

    with (
        instance_for_test(
            {
                "run_launcher": {
                    "class": "DockerRunLauncher",
                    "module": "dagster_docker",
                    "config": {},
                },
            }
        ) as instance,
        mock.patch("docker.client.from_env") as mock_docker_client_from_env,
    ):
        mock_docker_client = mock.MagicMock()
        mock_docker_client_from_env.return_value = mock_docker_client

        mock_container = mock.Mock()
        mock_container.attrs = {"State": mock_container_state, "Status": "exited"}
        mock_container.status = "exited"

        run_launcher = DockerRunLauncher()

        mock_containers = mock.MagicMock()
        mock_containers.get.return_value = mock_container

        # Mock containers.get to return the mock container
        mock_docker_client.containers = mock_containers

        run = create_run_for_test(
            instance,
            "test_job",
            status=DagsterRunStatus.STARTED,
            tags={DOCKER_CONTAINER_ID_TAG: "12345"},
        )

        health_check = run_launcher.check_run_worker_health(run)
        assert health_check.status == WorkerStatus.FAILED
        assert (
            health_check.msg
            == f"Container status is exited. Container state: {json.dumps(mock_container_state)}"
        )


@pytest.mark.integration
def test_terminate(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()
    launcher_config = {
        "env_vars": aws_env,
        "network": "container:postgres",
        "image": docker_image,
    }

    _test_launch(docker_postgres_instance, docker_image, launcher_config, terminate=True)


def _test_launch(
    instance_cm,
    docker_image,
    launcher_config,
    terminate=False,
    container_image=None,
    container_context=None,
):
    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_yamls(
        [
            os.path.join(get_test_project_environments_path(), "env.yaml"),
            os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
        ]
    )

    with instance_cm(
        overrides={
            "run_launcher": {
                "class": "DockerRunLauncher",
                "module": "dagster_docker",
                "config": launcher_config,
            }
        }
    ) as instance:
        recon_job = get_test_project_recon_job(
            "demo_job_s3", container_image=container_image, container_context=container_context
        )
        with get_test_project_workspace_and_remote_job(
            instance, "demo_job_s3", container_image=container_image
        ) as (
            workspace,
            orig_job,
        ):
            remote_job = ReOriginatedExternalJobForTest(orig_job)

            run = instance.create_run_for_job(
                job_def=recon_job.get_definition(),
                run_config=run_config,
                remote_job_origin=remote_job.get_remote_origin(),
                job_code_origin=recon_job.get_python_origin(),
            )

            instance.launch_run(run.run_id, workspace)

            if not terminate:
                poll_for_finished_run(instance, run.run_id, timeout=60)

                assert instance.get_run_by_id(run.run_id).status == DagsterRunStatus.SUCCESS
            else:
                start_time = time.time()

                filters = RunsFilter(
                    run_ids=[run.run_id],
                    statuses=[
                        DagsterRunStatus.STARTED,
                    ],
                )

                while True:
                    runs = instance.get_runs(filters, limit=1)
                    if runs:
                        break
                    else:
                        time.sleep(0.1)
                        if time.time() - start_time > 60:
                            raise Exception("Timed out waiting for run to start")

                launcher = instance.run_launcher
                assert launcher.terminate(run.run_id)

                poll_for_finished_run(instance, run.run_id, timeout=60)
                assert instance.get_run_by_id(run.run_id).status == DagsterRunStatus.CANCELED

                # termination is a no-op once run is finished
                assert not launcher.terminate(run.run_id)



import os
import time

import pytest
from dagster._core.definitions.events import AssetKey
from dagster._core.storage.dagster_run import DagsterRunStatus
from dagster._core.test_utils import poll_for_finished_run
from dagster._utils.merger import merge_dicts
from dagster_shared.yaml_utils import load_yaml_from_path, merge_yamls
from dagster_test.test_project import (
    ReOriginatedExternalJobForTest,
    find_local_test_image,
    get_buildkite_registry_config,
    get_test_project_docker_image,
    get_test_project_environments_path,
    get_test_project_recon_job,
    get_test_project_workspace_and_remote_job,
)

from dagster_docker_tests import IS_BUILDKITE


@pytest.mark.flaky(max_runs=2)
@pytest.mark.parametrize(
    "from_pending_repository, asset_selection",
    [
        (False, None),
        (True, None),
        (True, {AssetKey("foo"), AssetKey("bar")}),
    ],
)
@pytest.mark.integration
def test_image_on_job(
    monkeypatch, docker_postgres_instance, aws_env, from_pending_repository, asset_selection
):
    monkeypatch.setenv("IN_EXTERNAL_PROCESS", "yes")
    docker_image = get_test_project_docker_image()

    launcher_config = {
        "env_vars": aws_env,
        "networks": ["container:postgres"],
        "container_kwargs": {
            "auto_remove": True,
            "volumes": ["/var/run/docker.sock:/var/run/docker.sock"],
        },
    }

    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    executor_config = (
        {
            "execution": {"config": {}},
        }
        if not from_pending_repository
        else {}
    )

    env_yamls = [os.path.join(get_test_project_environments_path(), "env_s3.yaml")]
    if not from_pending_repository:
        env_yamls.append(os.path.join(get_test_project_environments_path(), "env.yaml"))
    run_config = merge_dicts(
        merge_yamls(env_yamls),
        executor_config,
    )

    with docker_postgres_instance(
        overrides={
            "run_launcher": {
                "class": "DockerRunLauncher",
                "module": "dagster_docker",
                "config": launcher_config,
            }
        }
    ) as instance:
        filename = "pending_repo.py" if from_pending_repository else "repo.py"
        recon_job = get_test_project_recon_job("demo_job_docker", docker_image, filename=filename)
        repository_load_data = recon_job.repository.get_definition().repository_load_data
        recon_job = recon_job.with_repository_load_data(repository_load_data)

        with get_test_project_workspace_and_remote_job(
            instance,
            "demo_job_docker",
            container_image=docker_image,
            filename=filename,
        ) as (
            workspace,
            orig_job,
        ):
            remote_job = ReOriginatedExternalJobForTest(
                orig_job, container_image=docker_image, filename=filename
            )

            run = instance.create_run_for_job(
                job_def=recon_job.get_definition(),
                run_config=run_config,
                remote_job_origin=remote_job.get_remote_origin(),
                job_code_origin=remote_job.get_python_origin(),
                repository_load_data=repository_load_data,
                asset_selection=frozenset(asset_selection) if asset_selection else None,
            )

            instance.launch_run(run.run_id, workspace)

            poll_for_finished_run(instance, run.run_id, timeout=60)

            for log in instance.all_logs(run.run_id):
                print(log)  # noqa: T201

            assert instance.get_run_by_id(run.run_id).status == DagsterRunStatus.SUCCESS


@pytest.mark.integration
def test_container_context_on_job(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()

    launcher_config = {}

    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    executor_config = {
        "execution": {"config": {}},
    }

    run_config = merge_dicts(
        merge_yamls(
            [
                os.path.join(get_test_project_environments_path(), "env.yaml"),
                os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
            ]
        ),
        executor_config,
    )

    with docker_postgres_instance(
        overrides={
            "run_launcher": {
                "class": "DockerRunLauncher",
                "module": "dagster_docker",
                "config": launcher_config,
            }
        }
    ) as instance:
        recon_job = get_test_project_recon_job(
            "demo_job_docker",
            docker_image,
            container_context={
                "docker": {
                    "env_vars": aws_env,
                    "networks": ["container:postgres"],
                    "container_kwargs": {
                        "auto_remove": True,
                        "volumes": ["/var/run/docker.sock:/var/run/docker.sock"],
                    },
                }
            },
        )
        with get_test_project_workspace_and_remote_job(
            instance, "demo_job_docker", container_image=docker_image
        ) as (
            workspace,
            orig_job,
        ):
            remote_job = ReOriginatedExternalJobForTest(orig_job, container_image=docker_image)

            run = instance.create_run_for_job(
                job_def=recon_job.get_definition(),
                run_config=run_config,
                remote_job_origin=remote_job.get_remote_origin(),
                job_code_origin=recon_job.get_python_origin(),
            )

            instance.launch_run(run.run_id, workspace)

            poll_for_finished_run(instance, run.run_id, timeout=60)

            for log in instance.all_logs(run.run_id):
                print(log)  # noqa: T201

            assert instance.get_run_by_id(run.run_id).status == DagsterRunStatus.SUCCESS


@pytest.mark.integration
def test_recovery(docker_postgres_instance, aws_env):
    docker_image = get_test_project_docker_image()

    launcher_config = {
        "env_vars": aws_env,
        "networks": ["container:postgres"],
        "container_kwargs": {
            "auto_remove": True,
            "volumes": ["/var/run/docker.sock:/var/run/docker.sock"],
        },
    }

    if IS_BUILDKITE:
        launcher_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_dicts(
        load_yaml_from_path(os.path.join(get_test_project_environments_path(), "env_s3.yaml")),  # pyright: ignore[reportArgumentType]
        {
            "ops": {
                "multiply_the_word_slow": {
                    "inputs": {"word": "bar"},
                    "config": {"factor": 2, "sleep_time": 10},
                }
            },
            "execution": {"config": {}},
        },
    )

    with docker_postgres_instance(
        overrides={
            "run_launcher": {
                "class": "DockerRunLauncher",
                "module": "dagster_docker",
                "config": launcher_config,
            },
            "run_monitoring": {"enabled": True, "max_resume_run_attempts": 3},
        }
    ) as instance:
        recon_job = get_test_project_recon_job("demo_slow_job_docker", docker_image)
        with get_test_project_workspace_and_remote_job(
            instance, "demo_slow_job_docker", container_image=docker_image
        ) as (
            workspace,
            orig_job,
        ):
            remote_job = ReOriginatedExternalJobForTest(orig_job, container_image=docker_image)

            run = instance.create_run_for_job(
                job_def=recon_job.get_definition(),
                run_config=run_config,
                remote_job_origin=remote_job.get_remote_origin(),
                job_code_origin=remote_job.get_python_origin(),
            )

            instance.launch_run(run.run_id, workspace)

            start_time = time.time()
            while time.time() - start_time < 60:
                run = instance.get_run_by_id(run.run_id)
                if run.status == DagsterRunStatus.STARTED:
                    break
                assert run.status == DagsterRunStatus.STARTING
                time.sleep(1)

            time.sleep(3)

            instance.run_launcher._get_container(  # noqa: SLF001
                instance.get_run_by_id(run.run_id)
            ).stop()
            instance.resume_run(run.run_id, workspace, attempt_number=1)
            poll_for_finished_run(instance, run.run_id, timeout=60)

            for log in instance.all_logs(run.run_id):
                print(str(log) + "\n")  # noqa: T201
            assert instance.get_run_by_id(run.run_id).status == DagsterRunStatus.SUCCESS



import os

import pytest
from dagster._core.execution.api import execute_job
from dagster._core.test_utils import environ
from dagster._utils.merger import merge_dicts
from dagster_shared.yaml_utils import merge_yamls
from dagster_test.test_project import (
    find_local_test_image,
    get_buildkite_registry_config,
    get_test_project_docker_image,
    get_test_project_environments_path,
    get_test_project_recon_job,
)

from dagster_docker_tests import IS_BUILDKITE
from dagster_docker_tests.test_launch_docker import check_event_log_contains


@pytest.mark.integration
def test_docker_executor_success(docker_postgres_instance, aws_env):
    """Note that this test relies on having AWS credentials in the environment."""
    executor_config = {
        "execution": {"config": {"networks": ["container:postgres"], "env_vars": aws_env}}
    }

    docker_image = get_test_project_docker_image()
    if IS_BUILDKITE:
        executor_config["execution"]["config"]["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_dicts(
        merge_yamls(
            [
                os.path.join(get_test_project_environments_path(), "env.yaml"),
                os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
            ]
        ),
        executor_config,
    )

    with environ({"DOCKER_LAUNCHER_NETWORK": "container:postgres"}):
        with docker_postgres_instance() as instance:
            recon_job = get_test_project_recon_job("demo_job_docker", docker_image)
            with execute_job(recon_job, run_config=run_config, instance=instance) as result:
                assert result.success


@pytest.mark.integration
def test_docker_executor_check_step_health(docker_postgres_instance, aws_env):
    executor_config = {
        "execution": {"config": {"networks": ["container:postgres"], "env_vars": aws_env}}
    }

    docker_image = get_test_project_docker_image()
    if IS_BUILDKITE:
        executor_config["execution"]["config"]["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_dicts(
        merge_yamls(
            [
                os.path.join(get_test_project_environments_path(), "env.yaml"),
                os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
            ]
        ),
        executor_config,
    )

    # force a segfault to terminate the container unexpectedly, step health check should then fail the step
    run_config["ops"]["multiply_the_word"]["config"]["should_segfault"] = True

    with environ({"DOCKER_LAUNCHER_NETWORK": "container:postgres"}):
        with docker_postgres_instance() as instance:
            recon_job = get_test_project_recon_job("demo_job_docker", docker_image)
            with execute_job(recon_job, run_config=run_config, instance=instance) as result:
                assert not result.success
                run_logs = instance.all_logs(result.run_id)

                check_event_log_contains(
                    run_logs,
                    [
                        (
                            "STEP_UP_FOR_RETRY",
                            'Execution of step "multiply_the_word" failed and has requested a retry',
                        ),
                        (
                            "STEP_RESTARTED",
                            'Started re-execution (attempt # 2) of step "multiply_the_word"',
                        ),
                        (
                            "STEP_UP_FOR_RETRY",
                            'Execution of step "multiply_the_word" failed and has requested a retry',
                        ),
                        (
                            "STEP_RESTARTED",
                            'Started re-execution (attempt # 3) of step "multiply_the_word"',
                        ),
                        (
                            "STEP_FAILURE",
                            'Execution of step "multiply_the_word" failed.',
                        ),
                    ],
                )


@pytest.mark.integration
def test_docker_executor_job_that_fails_run_worker(docker_postgres_instance, aws_env):
    executor_config = {
        "execution": {"config": {"networks": ["container:postgres"], "env_vars": aws_env}}
    }

    docker_image = get_test_project_docker_image()
    if IS_BUILDKITE:
        executor_config["execution"]["config"]["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_dicts(
        merge_yamls(
            [
                #                os.path.join(get_test_project_environments_path(), "env.yaml"),
                os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
            ]
        ),
        executor_config,
    )

    with environ({"DOCKER_LAUNCHER_NETWORK": "container:postgres"}):
        with docker_postgres_instance() as instance:
            recon_job = get_test_project_recon_job("fails_run_worker_job_docker", docker_image)
            with execute_job(recon_job, run_config=run_config, instance=instance) as result:
                assert not result.success
                run_logs = instance.all_logs(result.dagster_run.run_id)

                check_event_log_contains(
                    run_logs,
                    [
                        (
                            "ENGINE_EVENT",
                            "Unexpected exception while steps were still in-progress - terminating running steps:",
                        ),
                        (
                            "STEP_FAILURE",
                            'Execution of step "hanging_op" failed.',
                        ),
                        (
                            "PIPELINE_FAILURE",
                            'Execution of run for "fails_run_worker_job_docker" failed. An exception was thrown during execution.',
                        ),
                    ],
                )


@pytest.mark.integration
def test_docker_executor_config_on_container_context(docker_postgres_instance, aws_env):
    """Note that this test relies on having AWS credentials in the environment."""
    executor_config = {"execution": {"config": {}}}

    docker_image = get_test_project_docker_image()
    if IS_BUILDKITE:
        executor_config["execution"]["config"]["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_dicts(
        merge_yamls(
            [
                os.path.join(get_test_project_environments_path(), "env.yaml"),
                os.path.join(get_test_project_environments_path(), "env_s3.yaml"),
            ]
        ),
        executor_config,
    )

    with environ({"DOCKER_LAUNCHER_NETWORK": "container:postgres"}):
        with docker_postgres_instance() as instance:
            recon_job = get_test_project_recon_job(
                "demo_job_docker",
                docker_image,
                container_context={
                    "docker": {
                        "networks": ["container:postgres"],
                        "env_vars": aws_env,
                    }
                },
            )
            with execute_job(recon_job, run_config=run_config, instance=instance) as result:
                assert result.success


@pytest.mark.integration
def test_docker_executor_retries(docker_postgres_instance, aws_env):
    """Note that this test relies on having AWS credentials in the environment."""
    executor_config = {
        "execution": {
            "config": {
                "networks": ["container:postgres"],
                "env_vars": aws_env,
                "retries": {"enabled": {}},
            }
        }
    }

    docker_image = get_test_project_docker_image()
    if IS_BUILDKITE:
        executor_config["execution"]["config"]["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    run_config = merge_dicts(
        merge_yamls([os.path.join(get_test_project_environments_path(), "env_s3.yaml")]),
        executor_config,
    )

    with environ({"DOCKER_LAUNCHER_NETWORK": "container:postgres"}):
        with docker_postgres_instance() as instance:
            recon_job = get_test_project_recon_job("step_retries_job_docker", docker_image)
            with execute_job(recon_job, run_config=run_config, instance=instance) as result:
                assert result.success



import docker
import pytest
from dagster import RetryRequested, job, op
from dagster_docker import docker_container_op, execute_docker_container
from dagster_docker.ops.docker_container_op import _get_container_name


def _get_container_logs(container_name):
    return str(docker.client.from_env().containers.get(container_name).logs())


def test_docker_container_op():
    first_op = docker_container_op.configured(
        {
            "image": "busybox",
            "entrypoint": ["/bin/sh", "-c"],
            "command": ["echo HI"],
        },
        name="first_op",
    )
    second_op = docker_container_op.configured(
        {
            "image": "busybox",
            "entrypoint": ["/bin/sh", "-c"],
            "command": ["echo GOODBYE"],
        },
        name="second_op",
    )

    @job
    def my_full_job():
        second_op(first_op())

    execute_result = my_full_job.execute_in_process()
    assert execute_result.success

    run_id = execute_result.dagster_run.run_id
    container_name = _get_container_name(run_id, first_op.name, 0)
    assert "HI" in _get_container_logs(container_name)

    container_name = _get_container_name(run_id, second_op.name, 0)
    assert "GOODBYE" in _get_container_logs(container_name)


def test_custom_docker_op():
    @op
    def my_custom_op(context):
        execute_docker_container(
            context,
            image="busybox",
            entrypoint=["/bin/sh", "-c"],
            command=["echo HI"],
        )
        return "GOODBYE"

    @op
    def my_second_custom_op(context, what_to_echo: str):
        execute_docker_container(
            context,
            image="busybox",
            entrypoint=["/bin/sh", "-c"],
            command=[f"echo {what_to_echo}"],
        )

    @job
    def my_job_with_custom_ops():
        my_second_custom_op(my_custom_op())

    execute_result = my_job_with_custom_ops.execute_in_process()
    assert execute_result.success

    run_id = execute_result.dagster_run.run_id
    job_name = _get_container_name(run_id, my_custom_op.name, 0)
    assert "HI" in _get_container_logs(job_name)

    job_name = _get_container_name(run_id, my_second_custom_op.name, 0)
    assert "GOODBYE" in _get_container_logs(job_name)


def test_docker_container_op_with_failure():
    failure_op = docker_container_op.configured(
        {
            "image": "busybox",
            "entrypoint": ["/bin/sh", "-c"],
            "command": ["exit 1"],
        },
        name="failure_op",
    )

    @job
    def failure_job():
        failure_op()

    with pytest.raises(Exception, match="Docker container returned exit code 1"):
        failure_job.execute_in_process()


def test_docker_container_op_retries():
    @op
    def fails_sometimes(context):
        execute_docker_container(
            context,
            image="busybox",
            entrypoint=["/bin/sh", "-c"],
            command=[f"echo HERE IS RETRY NUMBER {context.retry_number}"],
        )
        if context.retry_number == 0:
            raise RetryRequested(max_retries=1, seconds_to_wait=1)

    @job
    def fails_sometimes_job():
        fails_sometimes()

    execute_result = fails_sometimes_job.execute_in_process()
    run_id = execute_result.dagster_run.run_id
    container_name = _get_container_name(run_id, fails_sometimes.name, 0)
    assert "HERE IS RETRY NUMBER 0" in _get_container_logs(container_name)

    container_name = _get_container_name(run_id, fails_sometimes.name, 1)

    assert "HERE IS RETRY NUMBER 1" in _get_container_logs(container_name)



# ruff: isort: skip_file
# fmt: off
# start_marker
from dagster_docker import docker_container_op

from dagster import job

first_op = docker_container_op.configured(
    {
        "image": "busybox",
        "command": ["echo HELLO"],
    },
    name="first_op",
)
second_op = docker_container_op.configured(
    {
        "image": "busybox",
        "command": ["echo GOODBYE"],
    },
    name="second_op",
)

@job
def full_job():
    second_op(first_op())
# end_marker
# fmt: on


def test_docker_container_op():
    assert full_job



import os
import tempfile

import pytest
from dagster import AssetExecutionContext, asset, materialize
from dagster._core.pipes.utils import PipesFileContextInjector
from dagster_docker.pipes import PipesDockerClient
from dagster_test.test_project import (
    IS_BUILDKITE,
    find_local_test_image,
    get_buildkite_registry_config,
    get_test_project_docker_image,
)


@pytest.mark.integration
def test_default():
    docker_image = get_test_project_docker_image()

    ext_config = {}

    if IS_BUILDKITE:
        ext_config["registry"] = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    @asset
    def number_x(
        context: AssetExecutionContext,
        pipes_client: PipesDockerClient,
    ):
        return pipes_client.run(
            image=docker_image,
            command=[
                "python",
                "-m",
                "numbers_example.number_x",
            ],
            context=context,
            extras={"storage_root": "/tmp/", "multiplier": 2},
            env={"PYTHONPATH": "/modules/dagster-test/dagster_test/toys/external_execution/"},
        ).get_results()

    result = materialize(
        [number_x],
        resources={"pipes_client": PipesDockerClient(**ext_config)},
        raise_on_error=False,
    )
    assert result.success
    mats = result.asset_materializations_for_node(number_x.op.name)
    assert mats[0].metadata["foo"]


@pytest.mark.integration
def test_file_io():
    docker_image = get_test_project_docker_image()

    registry = None

    if IS_BUILDKITE:
        registry = get_buildkite_registry_config()
    else:
        find_local_test_image(docker_image)

    with tempfile.TemporaryDirectory() as tempdir:

        @asset
        def number_x(
            context: AssetExecutionContext,
            pipes_client: PipesDockerClient,
        ):
            instance_storage = context.instance.storage_directory()
            host_storage = os.path.join(instance_storage, "number_example")
            os.makedirs(host_storage, exist_ok=True)

            container_storage = "/mnt/asset_storage/"

            volumes = {
                host_storage: {
                    "bind": container_storage,
                    "mode": "rw",
                },
                tempdir: {
                    "bind": tempdir,
                    "mode": "rw",
                },
            }

            return pipes_client.run(
                image=docker_image,
                command=[
                    "python",
                    "-m",
                    "numbers_example.number_x",
                ],
                registry=registry,
                context=context,
                extras={"storage_root": container_storage, "multiplier": 2},
                container_kwargs={
                    "environment": {
                        "PYTHONPATH": "/modules/dagster-test/dagster_test/toys/external_execution/"
                    },
                    "volumes": volumes,
                },
            ).get_results()

        result = materialize(
            [number_x],
            resources={
                "pipes_client": PipesDockerClient(
                    context_injector=PipesFileContextInjector(os.path.join(tempdir, "context")),
                )
            },
            raise_on_error=False,
        )
        assert result.success
        mats = result.asset_materializations_for_node(number_x.op.name)
        assert mats[0].metadata["foo"]


_print_script = """
import sys
import time

sys.stdout.write('w1')
print('p1')
sys.stdout.write('w2')
print('p2')
"""


@pytest.mark.integration
def test_bytes_logs():
    # docker_image = get_test_project_docker_image()

    # ext_config = {}

    # if IS_BUILDKITE:
    #     ext_config["registry"] = get_buildkite_registry_config()
    # else:
    #     find_local_test_image(docker_image)

    @asset
    def bytes_logs(
        context: AssetExecutionContext,
    ):
        c = PipesDockerClient()

        return c.run(
            context=context,
            image="python:3.10-slim",
            command=[
                "python",
                "-c",
                _print_script,
            ],
        ).get_results()

    result = materialize(
        [bytes_logs],
        raise_on_error=False,
    )
    assert result.success
